Custom Tables is a powerful component/[plugin](https://ct4.us/) that allows you to add custom database tables, fields, and layouts to both Joomla and [WordPress](https://ct4.us/). Its versatility makes it suitable for a wide range of applications, including catalogs, edit forms, detail pages, and more.

One of the standout features of Custom Tables is its support for the modern Twig template language, providing a flexible and user-friendly way to create dynamic layouts.

With over 40 field types to choose from, including Integer, Decimal, Text String, Date, Time, Email, Color, Image, File, Table Join, User, and Language, you can create complex data structures tailored to your specific needs. The edit form input boxes dynamically adapt to the selected field type, ensuring a seamless user experience. For example, the Date field type will display a calendar, the Color field type will offer a color picker, and the Image field type will provide an image uploader.

Custom Tables also supports table connections using the Table Join field type, allowing you to create intricate relationships between your data using LEFT JOIN operations.

The Layout Editor with Twig language simplifies the process of creating new layouts by offering an Auto-Create button that generates a layout based on the list of fields in your table.

All tables are stored in MySQL, MariaDB, or PostgreSQL databases, with queries and field values being properly sanitized to prevent potential vulnerabilities for added security.

With its rich feature set and cross-platform compatibility, Custom Tables is an indispensable tool for developers and website administrators seeking to extend the functionality of their Joomla or [WordPress](https://ct4.us/) sites with custom data structures and dynamic layouts.

# How it works

**Create a table and fields**

Joomla:

![](https://joomlaboat.com/images/components/ct/listoftables.png)
![](https://joomlaboat.com/images/components/ct/listoffields.png)

WordPress:

![](https://joomlaboat.com/images/components/ct/listoftables_wp.png)
![](https://joomlaboat.com/images/components/ct/listoffields_wp.png)

**Create Edit Form and Catalog Layouts to show table content on the front-end.**

Joomla:

![](https://joomlaboat.com/images/components/ct/listoflayouts.png)
![](https://joomlaboat.com/images/components/ct/layout.png)

WordPress:

![](https://joomlaboat.com/images/components/ct/listoflayouts_wp.png)
![](https://joomlaboat.com/images/components/ct/layout_wp.png)


The layout is a template language for HTML code with special tags to output data from the table records.

When rendering catalog records, the tag will be replaced with the field value.

For example you have a field "producttitle", to output its value type this:

`<p>{{ producttitle }}</p>`

Where `<p></p>` is the HTML paragraph tag. and `{{ producttitle }}` is the Twig tag that will return the field "producttitle" value.

In Joomla, to display the **custom form** on the **front-end** of your website, you need to **create a new menu item** that links to the **form page**.

![](https://joomlaboat.com/images/components/ct/menu.png) Create Custom Table Catalog menu item, select table, catalog layout, edit form layout and configure other settings such as access level, etc.

Ready.

![](https://joomlaboat.com/images/components/ct/appointments.png)

In WordPress, to display the **custom form** on the **front-end**, create a new **Page** or **Post**, and place the **Custom Tables Block** within the content area.

![](https://joomlaboat.com/images/components/ct/block.png) Add Custom Table Block, select table, catalog layout, edit form layout.

Ready.